import React from "react";
import ReactDOM from "react-dom";
import Msal from "./Msal";

ReactDOM.render(
  <React.StrictMode>
    <Msal />
  </React.StrictMode>,
  document.getElementById("root")
);
